@@ -334,6 +334,8 @@ const Real Inf = DBL_MAX;
 
 #endif
 
+#define ABS(x) ((x) >= 0.0 ? (x) : -(x)) 
+
 Real str2R (const char *q, char **endptr)
 {
   return STRTOR (q, endptr);
@@ -426,7 +428,25 @@ void printno (Real u, int m)
 
 Real Cabs (Complex z)
 {
-  return SQRT (z.re * z.re + z.im * z.im);
+  Real a, b, q;
+
+  a = ABS (z.re);
+  if (z.im == 0.0)
+    return a;
+  else
+    b = ABS (z.im);
+  /* b > 0 ! */
+  if (b > a)
+    {
+      q = a / b;
+      return b * SQRT (1+q*q);
+    }
+  else
+    {
+      /* a >= b ===> a > 0 */
+      q = b / a;
+      return a * SQRT (1+q*q);
+    }
 }
 
 Complex  Csub (Complex z1, Complex z2)
